/*****************************************************************************
*   Presentation: 300 - Using Event Notifications in SQL Server 2005-2008
*   FileName:  	  05 - Building an Activation Procedure.sql
*
*   Summary: Demonstrates building a activation stored procedure to automate
*            processing of queued events from Event Notifications.
*
*   Date: September 7, 2011 
*
*   SQL Server Versions:
*         2005, 2008, 2008 R2
*         
******************************************************************************
*   Copyright (C) 2011 Jonathan M. Kehayias, SQLskills.com
*   All rights reserved. 
*
*   For more scripts and sample code, check out 
*      http://sqlskills.com/blogs/jonathan
*
*   You may alter this code for your own *non-commercial* purposes. You may
*   republish altered code as long as you include this copyright and give 
*	due credit. 
*
*
*   THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
*   ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
*   TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
*   PARTICULAR PURPOSE. 
*
******************************************************************************/

USE [EventNotificationsDB]
GO

-- Create an Activation Procedure for the Queue
CREATE PROCEDURE [dbo].[ProcessEventNotifications]
WITH EXECUTE AS OWNER
AS 
SET NOCOUNT ON
DECLARE @message_body xml 
DECLARE @email_message nvarchar(MAX)
WHILE (1 = 1)
BEGIN
	BEGIN TRANSACTION
	-- Receive the next available message FROM the queue
	WAITFOR (
		RECEIVE TOP(1) -- just handle one message at a time
			@message_body=message_body
			FROM dbo.EventNotificationQueue
	), TIMEOUT 1000  -- if the queue is empty for one second, give UPDATE and go away
	-- If we didn't get anything, bail out
	IF (@@ROWCOUNT = 0)
		BEGIN
			ROLLBACK TRANSACTION
			BREAK
		END 

	IF (@message_body.value('(/EVENT_INSTANCE/Severity)[1]', 'int' ) > 10) -- Error is not Informational
	BEGIN

	-- Add the ServerName to the email subject
	DECLARE @subject NVARCHAR(500) = 'Demo: '+@@SERVERNAME+' SQL Server Error Log Event'
	
	-- Generate formatted email message
	SELECT @email_message = 'The following event was logged in the SQL Server ErrorLog:' + CHAR(10) +
	'PostTime: ' + @message_body.value('(/EVENT_INSTANCE/PostTime)[1]', 'varchar(128)') + CHAR(10) +
	'Error: ' + @message_body.value('(/EVENT_INSTANCE/Error)[1]', 'varchar(20)' )  + CHAR(10) +
	'Severity: ' + @message_body.value('(/EVENT_INSTANCE/Severity)[1]', 'varchar(20)' ) + CHAR(10) +
	'TextData: ' + @message_body.value('(/EVENT_INSTANCE/TextData)[1]', 'varchar(4000)' );
	-- Send email using Database Mail
    EXEC msdb.dbo.sp_send_dbmail
             @profile_name = 'DBA Notification Account', -- your defined email profile 
             @recipients = 'jmkehayias@gmail.com', -- your email
             @subject = @subject,
             @body = @email_message;

	END
--  Commit the transaction.  At any point before this, we could roll 
--  back - the received message would be back on the queue AND the response
--  wouldn't be sent.
	COMMIT TRANSACTION
END
GO

-- Check the Queue
SELECT *
FROM EventNotificationQueue
ORDER BY queuing_order;
GO

-- Generate Multiple Events
RAISERROR (N'Test ERRORLOG Event Notifications', 10, 1) WITH LOG;
GO
RAISERROR (N'Test ERRORLOG Event Notifications', 16, 1) WITH LOG;
GO

-- Test the Stored Procedure  (This should fail due to cross db access for sp_send_dbmail!)
EXECUTE [dbo].[ProcessEventNotifications]

